# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import urllib
import os.path
import re
import string
import sys
import time

import Version
import HRFData

FETCH_ERROR_STRANGE_RESPONSE = 1
FETCH_ERROR_LOGIN_ERROR = 2
FETCH_ERROR_PLAYING_MATCH = 3
FETCH_ERROR_ERRONEOUS_DATA = 4
FETCH_ERROR_COULDNT_GET_COOKIE = 5

class MattricksURLopener(urllib.FancyURLopener):
    def __init__(self, *args):
        self.version = "Mattricks/" + Version.version
        urllib.FancyURLopener.__init__(self, *args)

urllib._urlopener = MattricksURLopener()

def make_hrf_filename(path, always_download, overwrite_old):
    def make_real_path(path, amount_numbers):
        filename = string.join(["%04d", "-%02d", "-%02d", \
                                "_%02d", "-%02d", "-%02d"][:amount_numbers], "") % \
                                time.localtime(time.time())[:amount_numbers] + ".hrf"
        return os.path.realpath(os.path.join(os.path.expanduser(path), filename))

    filename = make_real_path(path, 3)
    if os.path.exists(filename):
        if not always_download:
            return
        elif overwrite_old:
            return filename
        else:
            filename = make_real_path(path, 6)
    return filename

user_cookie = (None, None)

def fetch_hrf(loginname, password, path = "",
              dlg=None, filename=None,
              always_download=True, overwrite_old=False, server="195.149.159.156"):
    global user_cookie

    if filename == None:
        filename = make_hrf_filename(path, always_download, overwrite_old)

    if filename == None:
        return None
    
    if user_cookie == (None, None) or user_cookie[0] != loginname:
        # ppna login-sidan, med loginnamn och lsenord.
        u = urllib.urlopen("http://" + server + "/Common/default.asp", \
                           urllib.urlencode([("loginname", loginname), ("password", password), \
                                             ("action", "login"), ("submit", "Log on")]))

        if dlg != None:
            dlg.Update(1)

        # Se till att lagra kakan
        try:
            cookie = u.info()['set-cookie']
            cookie = cookie[:string.find(cookie, ";")]
        except:
            return FETCH_ERROR_COULDNT_GET_COOKIE

        regexp = re.compile("loginError=(Yes|)&", re.IGNORECASE)
        pwdata = u.read()
        result = regexp.search(pwdata)
        if not result:
            print pwdata
            return FETCH_ERROR_STRANGE_RESPONSE
        if len(result.group(1)) > 0:
            #print pwdata
            return FETCH_ERROR_LOGIN_ERROR

        user_cookie = (loginname, cookie)
    else:
        cookie = user_cookie[1]

    if dlg != None:
        dlg.Update(2)

    o = MattricksURLopener()

    if dlg != None:
        dlg.Update(5, "Downloading HRF-file...")

    o.addheader("Cookie", cookie)
    u = o.open("http://" + server +"/Common/hrf.asp")
    content = u.read()

    m = HRFData.HTDir.re_date.search(content)
    if m:
        open(filename, 'w').write(content)
    else:
        m = re.compile("playingMatch=true").search(content)
        if m:
            return FETCH_ERROR_PLAYING_MATCH
        else:
            return FETCH_ERROR_ERRONEOUS_DATA

    if dlg != None:
        dlg.Update(9, "Updating tables...")

    return filename
