# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import os.path
import time
import sys

from Config import config
import Errorwx
import HRFData
import AboutPage
import Playerlist
import ComparisonPage
import Fetchdialog
import Fetch
import EconomyPage
import LineupPage
import MatchPrediction
import SummaryPage
import JuniorSpy
import TeamSelectorPage
import Configdialog

from wxPython.wx import *
from Columnlists import *
from Common import *
from Commonwx import *

class MainFrame(wxFrame):
    ID_FETCH = 101
    ID_REFETCH = 102
    ID_EXIT  = 103
    ID_CONFIG = 104
    ID_OPEN = 105

    def __init__(self, parent, ID, title, filename):
        wxFrame.__init__(self, parent, ID, title)

        if filename != None:
            config.set("Fetch", "save directory", os.path.dirname(filename))

        self.default_pass = ""
        self.default_user = ""

        sizer = wxBoxSizer(wxVERTICAL)

        hsizer = wxBoxSizer(wxHORIZONTAL)

        buttons = [(MainFrame.ID_FETCH, "Ladda ner data"),
                   (MainFrame.ID_REFETCH, "Ladda ner data igen"),
                   (MainFrame.ID_CONFIG, "Instllningar"),
                   (MainFrame.ID_OPEN, "ppna datafil"),
                   (MainFrame.ID_EXIT, "Avsluta")]
        for id, info in buttons:
            button = wxButton(self, id, info)
            hsizer.Add(button, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        sizer.AddSizer(hsizer, 0, wxEXPAND | wxBOTTOM, 0)

        notebook = wxNotebook(self, -1)
        nbsizer = wxNotebookSizer(notebook)

        the_pages = [
            ("about", "Mattricks", AboutPage.AboutPage(notebook, -1)),
            ("summary", "versikt", SummaryPage.SummaryPage(notebook, -1, filename)),
            ("player", "Spelarlista",
             Playerlist.PlayerPage(notebook, -1, filename, player_columns)),
            ("goal", "Skytteliga",
             Playerlist.PlayerPage(notebook, -1, filename, goal_columns)),
            ("comparison", "Jmfrelser", ComparisonPage.ComparisonPage(notebook, -1)),
            ("economy", "Ekonomi", EconomyPage.EconomyPage(notebook, -1, filename)),
            ("lineup", "Laguppstllning", LineupPage.LineupPage(notebook, -1, filename)),
            ("lineupselect", "Laguttagning",
             TeamSelectorPage.TeamSelectorPage(notebook, -1, filename)),
            ("prediction", "Matchberkning", MatchPrediction.MatchPrediction(notebook, -1))
            ]
        self.pages = {}
        for name, descr, page in the_pages:
            notebook.AddPage(page, descr)
            self.pages[name] = page

        sizer.AddSizer(nbsizer, 1, wxEXPAND | wxALL, 0)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        EVT_BUTTON(self, MainFrame.ID_OPEN, self.OnOpen)
        EVT_BUTTON(self, MainFrame.ID_FETCH, self.OnFetch)
        EVT_BUTTON(self, MainFrame.ID_CONFIG, self.OnConfig)
        EVT_BUTTON(self, MainFrame.ID_REFETCH, self.OnRefetch)
        EVT_BUTTON(self, MainFrame.ID_EXIT,  self.OnExit)

    def AfterInit(self):
        self.default_dir = config.get("Fetch", "save directory")
        self.default_jspy = config.get("Fetch", "junior spy")
        self.default_adownload = config.get("Fetch", "always download")
        self.default_override = config.get("Fetch", "override old hrf")
        self.default_server = config.get("Fetch", "ip number")
        if config.get("Fetch", "download at startup"):
            self.__do_fetch(True,
                            self.default_dir, self.default_user, self.default_pass,
                            self.default_jspy, self.default_adownload, self.default_override,
                            self.default_server)
        return True

    def OnOpen(self, event):
        wildcard = "HRF Files (*.hrf)|*.hrf|" \
                   "All files (*.*)|*.*"

        dlg = wxFileDialog(self, "Vlj fil med hattrickdata", wildcard = wildcard,
                           style = wxOPEN, defaultDir = self.default_dir)
        if dlg.ShowModal() == wxID_OK:
            self.__change_hrf(dlg.GetPath())
        dlg.Destroy()

    def OnFetch(self, event):
        self.default_dir = config.get("Fetch", "save directory")
        self.default_jspy = config.get("Fetch", "junior spy")
        self.default_adownload = config.get("Fetch", "always download")
        self.default_override = config.get("Fetch", "override old hrf")
        self.default_server = config.get("Fetch", "ip number")
        self.__do_fetch(True,
                        self.default_dir, self.default_user, self.default_pass,
                        self.default_jspy, self.default_adownload, self.default_override,
                        self.default_server)

    def OnRefetch(self, event):
        self.default_dir = config.get("Fetch", "save directory")
        self.default_jspy = config.get("Fetch", "junior spy")
        self.default_adownload = config.get("Fetch", "always download")
        self.default_override = config.get("Fetch", "override old hrf")
        self.default_server = config.get("Fetch", "ip number")
        self.__do_fetch(False,
                        self.default_dir, self.default_user, self.default_pass,
                        self.default_jspy, self.default_adownload, self.default_override,
                        self.default_server)

    def OnExit(self, event):
        self.Close(True)

    def OnConfig(self, event):
        if not Configdialog.ConfigDialog.conf_dialog_running:
            dlg = Configdialog.ConfigDialog(self, -1)
            dlg.Show()

    def __change_hrf(self, hrffile):
        HRFData.htdirs.add_from_filename(hrffile)
        for name, page in self.pages.items():
            page.ChangeHRF(hrffile)

    def __do_fetch(self, use_dialog,
                   path, loginname, password, juniorspy,
                   always_download, overwrite_old, server):

        exit = False
        hrffile = None
        if use_dialog:
            dlg = Fetchdialog.FetchDialog(self, "Ladda ner data",
                                          defaultDir = self.default_dir,
                                          defaultPass = self.default_pass,
                                          defaultUser = self.default_user,
                                          defaultJSpy = self.default_jspy,
                                          defaultADownload = self.default_adownload,
                                          defaultOverride = self.default_override)
            while not exit:
                res = dlg.ShowModal()
                if res == wxID_OK:
                    args = dlg.GetResult() + (self.default_server,)
                    (dummy, dummy, dummy, juniorspy, dummy, dummy, dummy) = args
                    (exit, hrffile) = self.__do_fetch_internal(*args)
                else:
                    exit = True
            dlg.Destroy()
        else:
            (exit, hrffile) = self.__do_fetch_internal(self.default_dir,
                                                       self.default_user,
                                                       self.default_pass,
                                                       self.default_jspy,
                                                       self.default_adownload,
                                                       self.default_override,
                                                       self.default_server)

        if hrffile != None:
            if juniorspy == True:
                psdlg=JuniorSpy.JuniorSpySelect(self,HRFData.hrfstorage[hrffile])
                psdlg.ShowModal()
            self.__change_hrf(hrffile)

    def __do_fetch_internal(self,
                            path, loginname, password, juniorspy,
                            always_download, overwrite_old, server):
        hrffile = None
        exit = False
        err_message = None
        pddlg = None
        if len(loginname) == 0 or len(password) == 0:
            err_message = "Du mste skriva in bde anvndarnamn och lsenord"
        else:
            pddlg = wxProgressDialog("Fetching hrf-file...",
                                     "Logging in...",
                                     10,
                                     self,
                                     wxPD_APP_MODAL)

            self.default_user = loginname
            self.default_pass = password
            result = Fetch.fetch_hrf(loginname, password, path, pddlg, None,
                                     always_download, overwrite_old, server)

            if result == Fetch.FETCH_ERROR_STRANGE_RESPONSE:
                err_message = "Konstigt svar frn servern"
                exit = True
            elif result == Fetch.FETCH_ERROR_LOGIN_ERROR:
                err_message = "Misslyckades att logga in, frsk igen!"
            elif result == Fetch.FETCH_ERROR_PLAYING_MATCH:
                err_message = "Laget spelar fr nrvarande match, frsk igen senare!"
                exit = True
            elif result == Fetch.FETCH_ERROR_ERRONEOUS_DATA:
                err_message = "Konstiga data frn hattrick-servern!"
                exit = True
            elif result == Fetch.FETCH_ERROR_COULDNT_GET_COOKIE:
                err_message = "Unable to get cookie!"
                exit = True
            else:
                hrffile = result
                exit = True

        if pddlg != None:
            pddlg.Destroy()

        if err_message != None:
            mdlg = wxMessageDialog(self, err_message,
                                  "Fel", wxOK | wxICON_ERROR)
            mdlg.ShowModal()
            mdlg.Destroy()

        return (exit, hrffile)

class Main(wxApp):
    ID_FRAME = 201
    def __init__(self, filename = None):
        self.initial_file = filename
        wxApp.__init__(self)

    def OnInit(self):
        frame = MainFrame(NULL, Main.ID_FRAME, "Mattricks", self.initial_file)
        frame.Show(true)

        Errorwx.error.init_errorwx(frame)

        self.SetTopWindow(frame)
        del self.initial_file

        frame.AfterInit()
        return true

def main(*hrfs):
    filename = None
    if len(hrfs) == 0:
        hrfs = config.get("General", "hrf directories")
    for hrf in hrfs:
        hrf = os.path.realpath(os.path.expanduser(hrf))
        if os.path.isdir(hrf):
            HRFData.htdirs.add_dirs(hrf)
        else:
            HRFData.htdirs.add_from_filename(hrf)
            filename = hrf

    if filename == None:
        filename = HRFData.htdirs.get_from_time(time.time())

    app = Main(filename)
    app.MainLoop()
