# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, os.path
import time

from Images import Images, create_star_image
import DataInfo
import HRFData
import Terms
import Columndialog
import SelectDateMenu
import PlayerDialog

from Common import *

from wxPython.wx import *
from wxPython.lib.mixins.listctrl import wxColumnSorterMixin, wxListCtrlAutoWidthMixin

class PlayerListCtrl(wxListCtrl, wxListCtrlAutoWidthMixin):
    def __init__(self, parent, ID, pos=wxDefaultPosition,
                 size=wxDefaultSize, style=0):
        wxListCtrl.__init__(self, parent, ID, pos, size, style)
        wxListCtrlAutoWidthMixin.__init__(self)

class PlayerListCtrlPanel(wxPanel):
    def __init__(self, parent, htdata, column_info):
        self.htdata = htdata
        self.column_info = column_info
        self.sort_column = -1
        self.sort_colname = ""

        wxPanel.__init__(self, parent, -1)

        tID = wxNewId()

        self.list = PlayerListCtrl(self, tID,
                                   style=wxLC_REPORT | wxSUNKEN_BORDER
                                   | wxLC_VRULES | wxLC_HRULES | wxLC_SINGLE_SEL
                                   )

        if wxPlatform != '__WXMSW__':
            self.imagelist = wxImageList(20, 12)
            self.images = {
                "up":   self.imagelist.Add(Images.get_bitmap_from_file("arrow_small_up.png")),
                "down": self.imagelist.Add(Images.get_bitmap_from_file("arrow_small_down.png")),
                "yellowcard": self.imagelist.Add(Images.get_bitmap_from_file("yellowcard.png")),
                "2yellowcards": self.imagelist.Add(Images.get_bitmap_from_file("2yellowcards.png")),
                "redcard": self.imagelist.Add(Images.get_bitmap_from_file("redcard.png"))
                }

            self.stars = {}
            for i in range(1,31):
                self.stars[i] = self.imagelist.Add(create_star_image(i))

            self.flags = {}
            for cid, cinfo in DataInfo.country_info.items():
                flagfile = cinfo[5]
                if flagfile != None:
                    bmp = Images.get_bitmap_from_file(flagfile)
                    self.flags[cid] = self.imagelist.Add(bmp)

            self.list.SetImageList(self.imagelist, wxIMAGE_LIST_SMALL)

        self.curr_col_list = []
        self.curr_hidden_list = []

        self.columns_increasing_sort = {}

        for id, (attribute, show_column, colflags, sort_increasing) in self.column_info.items():
            colname = DataInfo.known_player_data_info[attribute][1]
            self.columns_increasing_sort[id] = sort_increasing
            if show_column:
                self.curr_col_list.append(colname)
            else:
                self.curr_hidden_list.append(colname)

        self.UpdateColumns()

        EVT_SIZE(self, self.OnSize)
        #EVT_LIST_BEGIN_DRAG(self, tID, self.OnBeginDrag)
        #EVT_LIST_BEGIN_RDRAG(self, tID, self.OnBeginRDrag)
        #EVT_LIST_BEGIN_LABEL_EDIT(self, tID, self.OnBeginLabelEdit)
        #EVT_LIST_END_LABEL_EDIT(self, tID, self.OnEndLabelEdit)
        #EVT_LIST_DELETE_ITEM(self, tID, self.OnDeleteItem)
        #EVT_LIST_DELETE_ALL_ITEMS(self, tID, self.OnDeleteAllItems)
        #EVT_LIST_ITEM_SELECTED(self, tID, self.OnItemSelected)
        #EVT_LIST_ITEM_DESELECTED(self, tID, self.OnItemDeselected)
        #EVT_LIST_ITEM_ACTIVATED(self, tID, self.OnItemActivated)
        #EVT_LIST_ITEM_FOCUSED(self, tID, self.OnItemFocused)
        #EVT_LIST_ITEM_MIDDLE_CLICK(self, tID, self.OnItemMiddleClick)
        EVT_LIST_ITEM_RIGHT_CLICK(self, tID, self.OnItemRightClick)
        #EVT_LIST_KEY_DOWN(self, tID, self.OnKeyDown)
        #EVT_LIST_INSERT_ITEM(self, tID, self.OnInsertItem)
        EVT_LIST_COL_CLICK(self, tID, self.OnColClick)
        #EVT_LIST_COL_RIGHT_CLICK(self, tID, self.OnColRightClick)
        #EVT_LIST_COL_BEGIN_DRAG(self, tID, self.OnColBeginDrag)
        #EVT_LIST_COL_DRAGGING(self, tID, self.OnColDragging)
        #EVT_LIST_COL_END_DRAG(self, tID, self.OnColEndDrag)
        #EVT_LIST_CACHE_HINT(self, tID, self.OnCacheHint)

    def OnSize(self, event):
        w,h = self.GetClientSizeTuple()
        self.list.SetDimensions(0, 0, w, h)

    def OnItemRightClick(self, event):
        pid = self.pid_list[event.GetData()]
        dlg = PlayerDialog.PlayerDialog(self, self.htdata, pid)
        ret = dlg.Show()


    def OnColClick(self, event):
        #print "ColClick"
        self.SortList(event.GetColumn(), False)

    def PopulateList(self):
        if self.htdata == None:
            return

        self.pid_list=[]
        x = 0
        for pid in self.htdata.get_pids():
            self.pid_list.append(pid)
            self.list.InsertImageStringItem(x, getattr(self.htdata.get_player(pid, True), \
                                                       self.column_info[0][0]), -1)
            y = 0
            for id, (attribute, show_column, colflags, sort_increasing) in self.column_info.items():
                colname = DataInfo.known_player_data_info[attribute][1]
                if colname in self.curr_col_list:
                    cvalue = getattr(self.htdata.get_player(pid), attribute)
                    dvalue = getattr(self.htdata.get_player(pid, True), attribute)
                    if dvalue == None:
                        value = ""
                    else:
                        value = dvalue
                    if wxPlatform != '__WXMSW__':
                        if attribute == "warnings":
                            if 0 < cvalue <= 3:
                                img = self.images[{1: "yellowcard",
                                                   2: "2yellowcards",
                                                   3: "redcard"}[cvalue]]
                                self.list.SetStringItem(x, y, "", img)
                            else:
                                self.list.SetStringItem(x, y, "")
                        elif attribute == "CountryID":
                            cid = Terms.country[cvalue]
                            if cid in self.flags.keys():
                                img = self.flags[cid]
                                self.list.SetStringItem(x, y, "", img)
                            else:
                                self.list.SetStringItem(x, y, value)
                        elif attribute == "rating":
                            if 0 < cvalue <= 30:
                                img = self.stars[cvalue]
                                self.list.SetStringItem(x, y, value, img)
                            elif cvalue == 0:
                                self.list.SetStringItem(x, y, "")
                            else:
                                self.list.SetStringItem(x, y, value)
                        else:
                            self.list.SetStringItem(x, y, value)
                    else:
                        self.list.SetStringItem(x, y, value)
                    y += 1
            self.list.SetItemData(x, x)
            x += 1

    def SortList(self, column, resort):
        if column == -1:
            column = 0
        old_col = self.sort_column
        real_col = self.GetRealColIdx(column)
        curr_col = self.column_info[real_col]
        HRFData.player_sort_attribute = curr_col[0]
        self.sort_column = column
        self.sort_colname = DataInfo.known_player_data_info[self.column_info[real_col][0]][1]
        if not resort and old_col == column:
            self.ascending = not self.ascending
            self.columns_increasing_sort[real_col] = self.ascending
        elif old_col != column:
            self.ascending = self.columns_increasing_sort[real_col]
        self.list.SortItems(self.__column_sorter)
        self.__update_column_images(old_col)

    def UpdateColumns(self):
        self.list.ClearAll()
        all_cols = []
        info = wxListItem()
        info.m_mask = wxLIST_MASK_TEXT | wxLIST_MASK_FORMAT
        if wxPlatform != '__WXMSW__':
            info.m_mask |= wxLIST_MASK_IMAGE
        info.m_image = -1
        new_sortcol = -1
        i = 0
        for id, (attribute, show_column, colflags, sort_increasing) in self.column_info.items():
            colname = DataInfo.known_player_data_info[attribute][1]
            if colname in self.curr_col_list:
                if colflags == None:
                    info.m_format = 0
                else:
                    info.m_format = colflags
                info.m_text = colname
                self.list.InsertColumnInfo(id, info)
                all_cols.append(id)
                if self.sort_colname == colname:
                    new_sortcol = i
                i += 1

        self.PopulateList()
        self.SortList(new_sortcol, True)

        for i in range(len(all_cols)):
            self.list.SetColumnWidth(i, wxLIST_AUTOSIZE)

    def GetRealColIdx(self, idx):
        x = 0
        y = 0
        for id, (attribute, show_column, colflags, sort_increasing) in self.column_info.items():
            colname = DataInfo.known_player_data_info[attribute][1]
            if colname in self.curr_col_list:
                if (x == idx):
                    return y
                x += 1
            y += 1
        print "GetRealColIdx failed"
        return 0

    def __column_sorter(self, key1, key2):
        p1 = self.htdata.get_player(self.pid_list[key1])
        p2 = self.htdata.get_player(self.pid_list[key2])
        if p1 > p2:
            res = 1
        elif p1 < p2:
            res = -1
        else:
            res = 0

        if not self.ascending:
            res = -res

        return res

    def __update_column_images(self, old_col):
        if wxPlatform == '__WXMSW__':
            return

        if old_col != -1:
            self.list.ClearColumnImage(old_col)
        if self.sort_column != -1:
            img = self.images[{True: "down", False: "up"}[self.ascending]]
            self.list.SetColumnImage(self.sort_column, img)


class PlayerPage(wxPanel):
    ID_POPUP_MENU = 211
    ID_DATE_SELECTED = 212
    ID_EDIT_COLUMNS = 213

    def __init__(self, parent, ID, filename, column_info):
        if filename == None:
            self.htdata = (None, None)
        else:
            self.htdata = (filename, HRFData.hrfstorage[filename])

        self.column_info = column_info
        wxPanel.__init__(self, parent, ID)

        sizer = wxBoxSizer(wxVERTICAL)
        hsizer = wxBoxSizer(wxHORIZONTAL)
        vsizer = wxBoxSizer(wxVERTICAL)

        self.htbutton = wxButton(self, PlayerPage.ID_POPUP_MENU, \
                                 "ndra datum", style = wxBU_EXACTFIT)
        vsizer.Add(self.htbutton, 0, wxALIGN_CENTRE | wxALL, 5)

        button = wxButton(self, PlayerPage.ID_EDIT_COLUMNS, "Edit columns", \
                          style = wxBU_EXACTFIT)
        vsizer.Add(button, 0, wxALIGN_CENTRE | wxALL, 5)

        hsizer.AddSizer(vsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        self.current_hrf_info = wxStaticText(self, -1, self.__get_current_text())
        hsizer.Add(self.current_hrf_info, 1, wxEXPAND | wxALL, 5)

        sizer.AddSizer(hsizer, 0, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        self.playerlist = PlayerListCtrlPanel(self, self.htdata[1], self.column_info)
        sizer.Add(self.playerlist, 1, wxEXPAND | wxALL, 1)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        EVT_BUTTON(self, PlayerPage.ID_POPUP_MENU, self.OnPopupMenu)
        EVT_BUTTON(self, PlayerPage.ID_EDIT_COLUMNS, self.OnEditCol)

    def ChangeHRF(self, hrffile):
        self.htdata = (hrffile, HRFData.hrfstorage[hrffile])
        self.playerlist.htdata = self.htdata[1]
        self.playerlist.UpdateColumns()
        self.current_hrf_info.SetLabel(self.__get_current_text())

    def OnEditCol(self, event):
        #print "Edit col"
        dlg = Columndialog.ColumnDialog(self.playerlist.curr_col_list, \
                                        self.playerlist.curr_hidden_list)
        ret = dlg.ShowModal()
        if ret == wxID_OK:
            #print "OK returned"
            # We should now get the enabled strings.
            self.playerlist.curr_col_list = dlg.GetEnabledStrings()
            self.playerlist.curr_hidden_list = dlg.GetDisabledStrings()
            self.playerlist.UpdateColumns()
        dlg.Destroy()

    def OnPopupMenu(self, event):
        sdmenu = SelectDateMenu.SelectDateMenu(self, PlayerPage.ID_DATE_SELECTED, self.htdata[0])
        SelectDateMenu.EVT_DATE_SELECTED(sdmenu, PlayerPage.ID_DATE_SELECTED, \
                                         self.OnDateSelected)
        rect = self.htbutton.GetRect()
        self.PopupMenu(sdmenu, (rect[0], rect[1] + rect[3]))

    def OnDateSelected(self, event):
        self.ChangeHRF(event.GetHRFFile())

    def __get_current_text(self):
        return htdata_text(self.htdata)
